@@ -22,8 +22,8 @@ import struct
 import traceback
 
 from time import time
-from typing import Any
 from typing import Callable
+from typing import Protocol
 
 from solaar.i18n import _
 
@@ -32,19 +32,21 @@ from . import common
 from . import descriptors
 from . import desktop_notifications
 from . import diversion
-from . import hidpp10_constants
+from . import exceptions
 from . import hidpp20
 from . import hidpp20_constants
 from . import settings
+from . import settings_validator
 from . import special_keys
 from .hidpp10_constants import Registers
+from .hidpp20 import KeyFlag
+from .hidpp20 import MappingFlag
 from .hidpp20_constants import GestureId
 from .hidpp20_constants import ParamId
 
 logger = logging.getLogger(__name__)
 
 _hidpp20 = hidpp20.Hidpp20()
-_DK = hidpp10_constants.DEVICE_KIND
 _F = hidpp20_constants.SupportedFeature
 
 
@@ -177,7 +179,7 @@ class RegisterDpi(settings.Setting):
     description = _("Mouse movement sensitivity")
     register = Registers.MOUSE_DPI
     choices_universe = common.NamedInts.range(0x81, 0x8F, lambda x: str((x - 0x80) * 100))
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe}
 
 
@@ -251,7 +253,7 @@ class Backlight(settings.Setting):
     description = _("Set illumination time for keyboard.")
     feature = _F.BACKLIGHT
     choices_universe = common.NamedInts(Off=0, Varying=2, VeryShort=5, Short=10, Medium=20, Long=60, VeryLong=180)
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe}
 
 
@@ -285,7 +287,7 @@ class Backlight2(settings.Setting):
             backlight.write()
             return True
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             backlight = device.backlight
@@ -322,7 +324,7 @@ class Backlight2Level(settings.Setting):
                 device.backlight.write()
             return True
 
-    class validator_class(settings.RangeValidator):
+    class validator_class(settings_validator.RangeValidator):
         @classmethod
         def build(cls, setting_class, device):
             reply = device.feature_request(_F.BACKLIGHT2, 0x20)
@@ -334,7 +336,7 @@ class Backlight2Level(settings.Setting):
 class Backlight2Duration(settings.Setting):
     feature = _F.BACKLIGHT2
     min_version = 3
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     min_value = 1
     max_value = 600  # 10 minutes - actual maximum is 2 hours
     validator_options = {"byte_count": 2}
@@ -363,7 +365,7 @@ class Backlight2DurationHandsOut(Backlight2Duration):
     label = _("Backlight Delay Hands Out")
     description = _("Delay in seconds until backlight fades out with hands away from keyboard.")
     feature = _F.BACKLIGHT2
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     rw_options = {"field": "dho"}
 
 
@@ -372,7 +374,7 @@ class Backlight2DurationHandsIn(Backlight2Duration):
     label = _("Backlight Delay Hands In")
     description = _("Delay in seconds until backlight fades out with hands near keyboard.")
     feature = _F.BACKLIGHT2
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     rw_options = {"field": "dhi"}
 
 
@@ -381,7 +383,7 @@ class Backlight2DurationPowered(Backlight2Duration):
     label = _("Backlight Delay Powered")
     description = _("Delay in seconds until backlight fades out with external power.")
     feature = _F.BACKLIGHT2
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     rw_options = {"field": "dpow"}
 
 
@@ -391,7 +393,7 @@ class Backlight3(settings.Setting):
     description = _("Set illumination time for keyboard.")
     feature = _F.BACKLIGHT3
     rw_options = {"read_fnid": 0x10, "write_fnid": 0x20, "suffix": b"\x09"}
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     min_value = 0
     max_value = 1000
     validator_options = {"byte_count": 2}
@@ -455,7 +457,7 @@ class PointerSpeed(settings.Setting):
     label = _("Sensitivity (Pointer Speed)")
     description = _("Speed multiplier for mouse (256 is normal multiplier).")
     feature = _F.POINTER_SPEED
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     min_value = 0x002E
     max_value = 0x01FF
     validator_options = {"byte_count": 2}
@@ -502,7 +504,7 @@ class OnboardProfiles(settings.Setting):
     for i in range(1, 16):
         choices_universe[i] = f"Profile {i}"
         choices_universe[i + 0x100] = f"Read-Only Profile {i}"
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
 
     class rw_class:
         def __init__(self, feature):
@@ -526,7 +528,7 @@ class OnboardProfiles(settings.Setting):
                 profile_change(device, common.bytes2int(data_bytes))
             return result
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             headers = hidpp20.OnboardProfiles.get_profile_headers(device)
@@ -556,7 +558,7 @@ class ReportRate(settings.Setting):
     choices_universe[7] = "7ms"
     choices_universe[8] = "8ms"
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             # if device.wpid == '408E':
@@ -588,7 +590,7 @@ class ExtendedReportRate(settings.Setting):
     choices_universe[5] = "250us"
     choices_universe[6] = "125us"
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             reply = device.feature_request(_F.EXTENDED_ADJUSTABLE_REPORT_RATE, 0x10)
@@ -640,7 +642,7 @@ class ScrollRatchet(settings.Setting):
     description = _("Switch the mouse wheel between speed-controlled ratcheting and always freespin.")
     feature = _F.SMART_SHIFT
     choices_universe = common.NamedInts(**{_("Freespinning"): 1, _("Ratcheted"): 2})
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe}
 
 
@@ -683,7 +685,7 @@ class SmartShift(settings.Setting):
 
     min_value = rw_class.MIN_VALUE
     max_value = rw_class.MAX_VALUE
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
 
 
 class SmartShiftEnhanced(SmartShift):
@@ -730,7 +732,7 @@ class ReprogrammableKeys(settings.Settings):
             key_struct.remap(special_keys.CONTROL[common.bytes2int(data_bytes)])
             return True
 
-    class validator_class(settings.ChoicesMapValidator):
+    class validator_class(settings_validator.ChoicesMapValidator):
         @classmethod
         def build(cls, setting_class, device):
             choices = {}
@@ -899,7 +901,7 @@ class DivertKeys(settings.Settings):
         def read(self, device, key):
             key_index = device.keys.index(key)
             key_struct = device.keys[key_index]
-            return b"\x00\x00\x01" if "diverted" in key_struct.mapping_flags else b"\x00\x00\x00"
+            return b"\x00\x00\x01" if MappingFlag.DIVERTED in key_struct.mapping_flags else b"\x00\x00\x00"
 
         def write(self, device, key, data_bytes):
             key_index = device.keys.index(key)
@@ -907,7 +909,7 @@ class DivertKeys(settings.Settings):
             key_struct.set_diverted(common.bytes2int(data_bytes) != 0)  # not regular
             return True
 
-    class validator_class(settings.ChoicesMapValidator):
+    class validator_class(settings_validator.ChoicesMapValidator):
         def __init__(self, choices, key_byte_count=2, byte_count=1, mask=0x01):
             super().__init__(choices, key_byte_count, byte_count, mask)
 
@@ -927,20 +929,20 @@ class DivertKeys(settings.Settings):
             sliding = gestures = None
             choices = {}
             if device.keys:
-                for k in device.keys:
-                    if "divertable" in k.flags and "virtual" not in k.flags:
-                        if "raw XY" in k.flags:
-                            choices[k.key] = setting_class.choices_gesture
+                for key in device.keys:
+                    if KeyFlag.DIVERTABLE in key.flags and KeyFlag.VIRTUAL not in key.flags:
+                        if KeyFlag.RAW_XY in key.flags:
+                            choices[key.key] = setting_class.choices_gesture
                             if gestures is None:
                                 gestures = MouseGesturesXY(device, name="MouseGestures")
                             if _F.ADJUSTABLE_DPI in device.features:
-                                choices[k.key] = setting_class.choices_universe
+                                choices[key.key] = setting_class.choices_universe
                                 if sliding is None:
                                     sliding = DpiSlidingXY(
                                         device, name="DpiSliding", show_notification=desktop_notifications.show
                                     )
                         else:
-                            choices[k.key] = setting_class.choices_divert
+                            choices[key.key] = setting_class.choices_divert
             if not choices:
                 return None
             validator = cls(choices, key_byte_count=2, byte_count=1, mask=0x01)
@@ -983,7 +985,7 @@ class AdjustableDpi(settings.Setting):
     rw_options = {"read_fnid": 0x20, "write_fnid": 0x30}
     choices_universe = common.NamedInts.range(100, 4000, str, 50)
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             dpilist = produce_dpi_list(setting_class.feature, 0x10, 1, device, 0)
@@ -1011,9 +1013,9 @@ class ExtendedAdjustableDpi(settings.Setting):
     rw_options = {"read_fnid": 0x50, "write_fnid": 0x60}
     keys_universe = common.NamedInts(X=0, Y=1, LOD=2)
     choices_universe = common.NamedInts.range(100, 4000, str, 50)
-    choices_universe[0] = "LOW"
-    choices_universe[1] = "MEDIUM"
-    choices_universe[2] = "HIGH"
+    choices_universe[1] = "LOW"
+    choices_universe[2] = "MEDIUM"
+    choices_universe[3] = "HIGH"
     keys = common.NamedInts(X=0, Y=1, LOD=2)
 
     def write_key_value(self, key, value, save=True):
@@ -1024,7 +1026,7 @@ class ExtendedAdjustableDpi(settings.Setting):
         result = self.write(self._value, save)
         return result[key] if isinstance(result, dict) else result
 
-    class validator_class(settings.ChoicesMapValidator):
+    class validator_class(settings_validator.ChoicesMapValidator):
         @classmethod
         def build(cls, setting_class, device):
             reply = device.feature_request(setting_class.feature, 0x10, 0x00)
@@ -1107,12 +1109,12 @@ class SpeedChange(settings.Setting):
             if self.device.persister:
                 self.device.persister["_speed-change"] = currentSpeed
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             key_index = device.keys.index(special_keys.CONTROL.DPI_Change)
             key = device.keys[key_index] if key_index is not None else None
-            if key is not None and "divertable" in key.flags:
+            if key is not None and KeyFlag.DIVERTABLE in key.flags:
                 keys = [setting_class.choices_extra, key.key]
                 return cls(choices=common.NamedInts.list(keys), byte_count=2)
 
@@ -1126,7 +1128,7 @@ class DisableKeyboardKeys(settings.BitFieldSetting):
     _labels = {k: (None, _("Disables the %s key.") % k) for k in special_keys.DISABLE}
     choices_universe = special_keys.DISABLE
 
-    class validator_class(settings.BitFieldValidator):
+    class validator_class(settings_validator.BitFieldValidator):
         @classmethod
         def build(cls, setting_class, device):
             mask = device.feature_request(_F.KEYBOARD_DISABLE_KEYS, 0x00)[0]
@@ -1158,7 +1160,7 @@ class Multiplatform(settings.Setting):
     # the problem here is how to construct the right values for the rules Set GUI,
     # as, for example, the integer value for 'Windows' can be different on different devices
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             def _str_os_versions(low, high):
@@ -1166,11 +1168,11 @@ class Multiplatform(settings.Setting):
                     if version == 0:
                         return ""
                     elif version & 0xFF:
-                        return str(version >> 8) + "." + str(version & 0xFF)
+                        return f"{str(version >> 8)}.{str(version & 0xFF)}"
                     else:
                         return str(version >> 8)
 
-                return "" if low == 0 and high == 0 else " " + _str_os_version(low) + "-" + _str_os_version(high)
+                return "" if low == 0 and high == 0 else f" {_str_os_version(low)}-{_str_os_version(high)}"
 
             infos = device.feature_request(_F.MULTIPLATFORM)
             assert infos, "Oops, multiplatform count cannot be retrieved!"
@@ -1200,7 +1202,7 @@ class DualPlatform(settings.Setting):
     choices_universe[0x01] = "Android, Windows"
     feature = _F.DUALPLATFORM
     rw_options = {"read_fnid": 0x00, "write_fnid": 0x20}
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe}
 
 
@@ -1213,7 +1215,7 @@ class ChangeHost(settings.Setting):
     rw_options = {"read_fnid": 0x00, "write_fnid": 0x10, "no_reply": True}
     choices_universe = common.NamedInts(**{"Host " + str(i + 1): i for i in range(3)})
 
-    class validator_class(settings.ChoicesValidator):
+    class validator_class(settings_validator.ChoicesValidator):
         @classmethod
         def build(cls, setting_class, device):
             infos = device.feature_request(_F.CHANGE_HOST)
@@ -1226,7 +1228,7 @@ class ChangeHost(settings.Setting):
             choices = common.NamedInts()
             for host in range(0, numHosts):
                 paired, hostName = hostNames.get(host, (True, ""))
-                choices[host] = str(host + 1) + ":" + hostName if hostName else str(host + 1)
+                choices[host] = f"{str(host + 1)}:{hostName}" if hostName else str(host + 1)
             return cls(choices=choices, read_skip_byte_count=1) if choices and len(choices) > 1 else None
 
 
@@ -1325,7 +1327,7 @@ class Gesture2Gestures(settings.BitFieldWithOffsetAndMaskSetting):
     choices_universe = hidpp20_constants.GestureId
     _labels = _GESTURE2_GESTURES_LABELS
 
-    class validator_class(settings.BitFieldWithOffsetAndMaskValidator):
+    class validator_class(settings_validator.BitFieldWithOffsetAndMaskValidator):
         @classmethod
         def build(cls, setting_class, device, om_method=None):
             options = [g for g in device.gestures.gestures.values() if g.can_be_enabled or g.default_enabled]
@@ -1342,7 +1344,7 @@ class Gesture2Divert(settings.BitFieldWithOffsetAndMaskSetting):
     choices_universe = hidpp20_constants.GestureId
     _labels = _GESTURE2_GESTURES_LABELS
 
-    class validator_class(settings.BitFieldWithOffsetAndMaskValidator):
+    class validator_class(settings_validator.BitFieldWithOffsetAndMaskValidator):
         @classmethod
         def build(cls, setting_class, device, om_method=None):
             options = [g for g in device.gestures.gestures.values() if g.can_be_diverted]
@@ -1363,7 +1365,7 @@ class Gesture2Params(settings.LongSettings):
     _labels = _GESTURE2_PARAMS_LABELS
     _labels_sub = _GESTURE2_PARAMS_LABELS_SUB
 
-    class validator_class(settings.MultipleRangeValidator):
+    class validator_class(settings_validator.MultipleRangeValidator):
         @classmethod
         def build(cls, setting_class, device):
             params = _hidpp20.get_gestures(device).params.values()
@@ -1397,7 +1399,7 @@ class MKeyLEDs(settings.BitFieldSetting):
         def read(self, device):  # no way to read, so just assume off
             return b"\x00"
 
-    class validator_class(settings.BitFieldValidator):
+    class validator_class(settings_validator.BitFieldValidator):
         @classmethod
         def build(cls, setting_class, device):
             number = device.feature_request(setting_class.feature, 0x00)[0]
@@ -1455,7 +1457,7 @@ class PersistentRemappableAction(settings.Settings):
             v = ks.remap(data_bytes)
             return v
 
-    class validator_class(settings.ChoicesMapValidator):
+    class validator_class(settings_validator.ChoicesMapValidator):
         @classmethod
         def build(cls, setting_class, device):
             remap_keys = device.remap_keys
@@ -1494,7 +1496,7 @@ class Sidetone(settings.Setting):
     label = _("Sidetone")
     description = _("Set sidetone level.")
     feature = _F.SIDETONE
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     min_value = 0
     max_value = 100
 
@@ -1507,7 +1509,7 @@ class Equalizer(settings.RangeFieldSetting):
     rw_options = {"read_fnid": 0x20, "write_fnid": 0x30, "read_prefix": b"\x00"}
     keys_universe = []
 
-    class validator_class(settings.PackedRangeValidator):
+    class validator_class(settings_validator.PackedRangeValidator):
         @classmethod
         def build(cls, setting_class, device):
             data = device.feature_request(_F.EQUALIZER, 0x00)
@@ -1533,8 +1535,9 @@ class ADCPower(settings.Setting):
     label = _("Power Management")
     description = _("Power off in minutes (0 for never).")
     feature = _F.ADC_MEASUREMENT
+    min_version = 2  # documentation for version 1 does not mention this capability
     rw_options = {"read_fnid": 0x10, "write_fnid": 0x20}
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
     min_value = 0x00
     max_value = 0xFF
     validator_options = {"byte_count": 1}
@@ -1546,7 +1549,7 @@ class BrightnessControl(settings.Setting):
     description = _("Control overall brightness")
     feature = _F.BRIGHTNESS_CONTROL
     rw_options = {"read_fnid": 0x10, "write_fnid": 0x20}
-    validator_class = settings.RangeValidator
+    validator_class = settings_validator.RangeValidator
 
     def __init__(self, device, rw, validator):
         super().__init__(device, rw, validator)
@@ -1570,7 +1573,7 @@ class BrightnessControl(settings.Setting):
                     return reply
             return super().write(device, data_bytes)
 
-    class validator_class(settings.RangeValidator):
+    class validator_class(settings_validator.RangeValidator):
         @classmethod
         def build(cls, setting_class, device):
             reply = device.feature_request(_F.BRIGHTNESS_CONTROL)
@@ -1591,7 +1594,7 @@ class LEDControl(settings.Setting):
     feature = _F.COLOR_LED_EFFECTS
     rw_options = {"read_fnid": 0x70, "write_fnid": 0x80}
     choices_universe = common.NamedInts(Device=0, Solaar=1)
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe}
 
 
@@ -1605,12 +1608,11 @@ class LEDZoneSetting(settings.Setting):
     label = _("LED Zone Effects")
     description = _("Set effect for LED Zone") + "\n" + _("LED Control needs to be set to Solaar to be effective.")
     feature = _F.COLOR_LED_EFFECTS
-    color_field = {"name": _LEDP.color, "kind": settings.KIND.choice, "label": None, "choices": colors}
-    speed_field = {"name": _LEDP.speed, "kind": settings.KIND.range, "label": _("Speed"), "min": 0, "max": 255}
-    period_field = {"name": _LEDP.period, "kind": settings.KIND.range, "label": _("Period"), "min": 100, "max": 5000}
-    intensity_field = {"name": _LEDP.intensity, "kind": settings.KIND.range, "label": _("Intensity"), "min": 0, "max": 100}
-    ramp_field = {"name": _LEDP.ramp, "kind": settings.KIND.choice, "label": _("Ramp"), "choices": hidpp20.LEDRampChoices}
-    # form_field = {"name": _LEDP.form, "kind": settings.KIND.choice, "label": _("Form"), "choices": _hidpp20.LEDFormChoices}
+    color_field = {"name": _LEDP.color, "kind": settings.Kind.CHOICE, "label": None, "choices": colors}
+    speed_field = {"name": _LEDP.speed, "kind": settings.Kind.RANGE, "label": _("Speed"), "min": 0, "max": 255}
+    period_field = {"name": _LEDP.period, "kind": settings.Kind.RANGE, "label": _("Period"), "min": 100, "max": 5000}
+    intensity_field = {"name": _LEDP.intensity, "kind": settings.Kind.RANGE, "label": _("Intensity"), "min": 0, "max": 100}
+    ramp_field = {"name": _LEDP.ramp, "kind": settings.Kind.CHOICE, "label": _("Ramp"), "choices": hidpp20.LedRampChoice}
     possible_fields = [color_field, speed_field, period_field, intensity_field, ramp_field]
 
     @classmethod
@@ -1620,14 +1622,14 @@ class LEDZoneSetting(settings.Setting):
         for zone in infos.zones:
             prefix = common.int2bytes(zone.index, 1)
             rw = settings.FeatureRW(cls.feature, read_fnid, write_fnid, prefix=prefix, suffix=suffix)
-            validator = settings.HeteroValidator(
-                data_class=hidpp20.LEDEffectSetting, options=zone.effects, readable=infos.readable
+            validator = settings_validator.HeteroValidator(
+                data_class=hidpp20.LEDEffectSetting, options=zone.effects, readable=infos.readable and read_fnid is not None
             )
             setting = cls(device, rw, validator)
             setting.name = cls.name + str(int(zone.location))
             setting.label = _("LEDs") + " " + str(hidpp20.LEDZoneLocations[zone.location])
             choices = [hidpp20.LEDEffects[e.ID][0] for e in zone.effects if e.ID in hidpp20.LEDEffects]
-            ID_field = {"name": "ID", "kind": settings.KIND.choice, "label": None, "choices": choices}
+            ID_field = {"name": "ID", "kind": settings.Kind.CHOICE, "label": None, "choices": choices}
             setting.possible_fields = [ID_field] + cls.possible_fields
             setting.fields_map = hidpp20.LEDEffects
             settings_.append(setting)
@@ -1645,7 +1647,7 @@ class RGBControl(settings.Setting):
     feature = _F.RGB_EFFECTS
     rw_options = {"read_fnid": 0x50, "write_fnid": 0x50}
     choices_universe = common.NamedInts(Device=0, Solaar=1)
-    validator_class = settings.ChoicesValidator
+    validator_class = settings_validator.ChoicesValidator
     validator_options = {"choices": choices_universe, "write_prefix_bytes": b"\x01", "read_skip_byte_count": 1}
 
 
@@ -1657,7 +1659,7 @@ class RGBEffectSetting(LEDZoneSetting):
 
     @classmethod
     def build(cls, device):
-        return cls.setup(device, 0xE0, 0x10, b"\x01")
+        return cls.setup(device, None, 0x10, b"\x01")
 
 
 class PerKeyLighting(settings.Settings):
@@ -1723,7 +1725,7 @@ class PerKeyLighting(settings.Settings):
     class rw_class(settings.FeatureRWMap):
         pass
 
-    class validator_class(settings.ChoicesMapValidator):
+    class validator_class(settings_validator.ChoicesMapValidator):
         @classmethod
         def build(cls, setting_class, device):
             choices_map = {}
@@ -1735,14 +1737,14 @@ class PerKeyLighting(settings.Settings):
                     key = (
                         setting_class.keys_universe[i]
                         if i in setting_class.keys_universe
-                        else common.NamedInt(i, "KEY " + str(i))
+                        else common.NamedInt(i, f"KEY {str(i)}")
                     )
                     choices_map[key] = setting_class.choices_universe
             result = cls(choices_map) if choices_map else None
             return result
 
 
-SETTINGS = [
+SETTINGS: list[settings.Setting] = [
     RegisterHandDetection,  # simple
     RegisterSmoothScroll,  # simple
     RegisterSideScroll,  # simple
@@ -1803,7 +1805,95 @@ SETTINGS = [
 ]
 
 
-def check_feature(device, settings_class: settings.Setting) -> None | bool | Any:
+class SettingsProtocol(Protocol):
+    @property
+    def name(self):
+        ...
+
+    @property
+    def label(self):
+        ...
+
+    @property
+    def description(self):
+        ...
+
+    @property
+    def feature(self):
+        ...
+
+    @property
+    def register(self):
+        ...
+
+    @property
+    def kind(self):
+        ...
+
+    @property
+    def min_version(self):
+        ...
+
+    @property
+    def persist(self):
+        ...
+
+    @property
+    def rw_options(self):
+        ...
+
+    @property
+    def validator_class(self):
+        ...
+
+    @property
+    def validator_options(self):
+        ...
+
+    @classmethod
+    def build(cls, device):
+        ...
+
+    def val_to_string(self, value):
+        ...
+
+    @property
+    def choices(self):
+        ...
+
+    @property
+    def range(self):
+        ...
+
+    def _pre_read(self, cached, key=None):
+        ...
+
+    def read(self, cached=True):
+        ...
+
+    def _pre_write(self, save=True):
+        ...
+
+    def update(self, value, save=True):
+        ...
+
+    def write(self, value, save=True):
+        ...
+
+    def acceptable(self, args, current):
+        ...
+
+    def compare(self, args, current):
+        ...
+
+    def apply(self):
+        ...
+
+    def __str__(self):
+        ...
+
+
+def check_feature(device, settings_class: SettingsProtocol) -> None | bool | SettingsProtocol:
     if settings_class.feature not in device.features:
         return
     if settings_class.min_version > device.features.get_feature_version(settings_class.feature):
@@ -1817,44 +1907,66 @@ def check_feature(device, settings_class: settings.Setting) -> None | bool | Any
         logger.error(
             "check_feature %s [%s] error %s\n%s", settings_class.name, settings_class.feature, e, traceback.format_exc()
         )
-        return False  # differentiate from an error-free determination that the setting is not supported
+        raise e  # differentiate from an error-free determination that the setting is not supported
 
 
-# Returns True if device was queried to find features, False otherwise
-def check_feature_settings(device, already_known):
-    """Auto-detect device settings by the HID++ 2.0 features they have."""
+def check_feature_settings(device, already_known) -> bool:
+    """Auto-detect device settings by the HID++ 2.0 features they have.
+
+    Returns
+    -------
+    bool
+        True, if device was fully queried to find features, False otherwise.
+    """
     if not device.features or not device.online:
         return False
     if device.protocol and device.protocol < 2.0:
         return False
     absent = device.persister.get("_absent", []) if device.persister else []
-    newAbsent = []
+    new_absent = []
     for sclass in SETTINGS:
         if sclass.feature:
             known_present = device.persister and sclass.name in device.persister
             if not any(s.name == sclass.name for s in already_known) and (known_present or sclass.name not in absent):
-                setting = check_feature(device, sclass)
+                try:
+                    setting = check_feature(device, sclass)
+                except Exception as err:
+                    # on an internal HID++ error, assume offline and stop further checking
+                    if (
+                        isinstance(err, exceptions.FeatureCallError)
+                        and err.error == hidpp20_constants.ErrorCode.LOGITECH_ERROR
+                    ):
+                        logger.warning(f"HID++ internal error checking feature {sclass.name}: make device not present")
+                        device.online = False
+                        device.present = False
+                        return False
+                    else:
+                        logger.warning(f"ignore feature {sclass.name} because of error {err}")
+
                 if isinstance(setting, list):
                     for s in setting:
                         already_known.append(s)
-                    if sclass.name in newAbsent:
-                        newAbsent.remove(sclass.name)
+                    if sclass.name in new_absent:
+                        new_absent.remove(sclass.name)
                 elif setting:
                     already_known.append(setting)
-                    if sclass.name in newAbsent:
-                        newAbsent.remove(sclass.name)
+                    if sclass.name in new_absent:
+                        new_absent.remove(sclass.name)
                 elif setting is None:
-                    if sclass.name not in newAbsent and sclass.name not in absent and sclass.name not in device.persister:
-                        newAbsent.append(sclass.name)
-    if device.persister and newAbsent:
-        absent.extend(newAbsent)
+                    if sclass.name not in new_absent and sclass.name not in absent and sclass.name not in device.persister:
+                        new_absent.append(sclass.name)
+    if device.persister and new_absent:
+        absent.extend(new_absent)
         device.persister["_absent"] = absent
     return True
 
 
-def check_feature_setting(device, setting_name):
+def check_feature_setting(device, setting_name: str) -> settings.Setting | None:
     for sclass in SETTINGS:
         if sclass.feature and sclass.name == setting_name and device.features:
-            setting = check_feature(device, sclass)
+            try:
+                setting = check_feature(device, sclass)
+            except Exception:
+                return None
             if setting:
                 return setting
